/*****************************************************************************

Xprotolab: XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
March 2011

Copyright 2010 Gabriel Anzziani

This program is distributed under the terms of the GNU General Public License 

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "main.h"
#include "awg.h"

// Amplitude, Type, Duty cycle, desired F, Offset, cycles
AWG_PARAM EEMEM EEAWG = { 1, -65, 1, 256, 103000, 0 };

uint8_t  EEMEM EEwave[256] = {      // Custom AWG, preset with exponential
    -117,-107, -97, -87, -78, -69, -61, -53, -45, -38, -31, -24, -18, -12,  -6,   0,
       5,  11,  16,  20,  25,  29,  33,  37,  41,  45,  49,  52,  55,  58,  61,  64,
      67,  69,  72,  74,  76,  79,  81,  83,  85,  87,  88,  90,  92,  93,  95,  96,
      97,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 111, 112,
     113, 113, 114, 115, 115, 116, 116, 117, 117, 118, 118, 118, 119, 119, 120, 120,
     120, 121, 121, 121, 122, 122, 122, 122, 123, 123, 123, 123, 123, 124, 124, 124,
     124, 124, 124, 125, 125, 125, 125, 125, 125, 125, 126, 126, 126, 126, 126, 126,
     126, 126, 126, 126, 126, 126, 127, 127, 127, 127, 127, 127, 127, 127, 127, 117,
     107,  97,  87,  78,  69,  61,  53,  45,  38,  31,  24,  18,  12,   6,   0,  -5,
     -11, -16, -20, -25, -29, -33, -37 ,-41, -45, -49, -52, -55, -58, -61, -64, -67,
     -69, -72, -74, -76, -79, -81, -83, -85, -87, -88, -90, -92, -93, -95, -96, -97,
     -99,-100,-101,-102,-103,-104,-105,-106,-107,-108,-109,-110,-111,-111,-112,-113,
    -113,-114,-115,-115,-116,-116,-117,-117,-118,-118,-118,-119,-119,-120,-120,-120,
    -121,-121,-121,-122,-122,-122,-122,-123,-123,-123,-123,-123,-124,-124,-124,-124,
    -124,-124,-125,-125,-125,-125,-125,-125,-125,-126,-126,-126,-126,-126,-126,-126,
    -126,-126,-126,-126,-126,-127,-127,-127,-127,-127,-127,-127,-127,-127,-127,-127,
};

// Global AWG variable
AWG_PARAM AWG;

void increaseF() {
         if(AWG.desiredF>6239999) AWG.desiredF =6250000;  // Maximum Freq=62.5KHz
    else if(AWG.desiredF>1000000) AWG.desiredF+=10000;
    else if(AWG.desiredF>100000)  AWG.desiredF+=1000;
    else if(AWG.desiredF>10000)   AWG.desiredF+=100;
    else if(AWG.desiredF>1000)    AWG.desiredF+=10;
    else                          AWG.desiredF+=1;
}

void decreaseF() {
         if(AWG.desiredF< 192)      AWG.desiredF = 191;  // Minimum Freq= 1.91Hz
    else if(AWG.desiredF<=1000)     AWG.desiredF-=1;
    else if(AWG.desiredF<=10000)    AWG.desiredF-=10;
    else if(AWG.desiredF<=100000)   AWG.desiredF-=100;
    else if(AWG.desiredF<=1000000)  AWG.desiredF-=1000;
    else                            AWG.desiredF-=10000;
}

void LoadAWGvars(void) {
    eeprom_read_block(&AWG, &EEAWG, sizeof(AWG_PARAM));
    if(AWG.amp>0) AWG.amp=0;                              // AWGAmp must be negative
    if(AWG.duty>512) AWG.duty = 256;                      // Duty cycle must be inside [0,512]
}

void SaveAWGvars(void) {
    eeprom_write_block(&AWG, &EEAWG, sizeof(AWG_PARAM));
}

void BuildWave(void) {
    uint8_t i,j;
    uint16_t step;          // used in duty cycle calculation
    uint16_t d;             // used in duty cycle calculation
    int8_t *AWGTemp1;       // AWG Temp1
    int8_t *AWGTemp2;       // AWG Temp2

    AWGTemp1 = bigtemp+512; // Use the big temp buffer
    AWGTemp2 = bigtemp+768;

    // Find Period and number of cycles depending on the desired frequency
    if(AWG.desiredF<97656) {
        AWG.cycles = 1;
    }
    else if(AWG.desiredF<195313) {
        AWG.cycles = 2;
    }
    else if(AWG.desiredF<390625) {
        AWG.cycles = 4;
    }
    else if(AWG.desiredF<781250) {
        AWG.cycles = 8;
    }
    else AWG.cycles = 16;
    // Construct waveform
    i=0;
    switch(AWG.type) {
        case 1: // Sine
            do { AWGTemp1[i] = Sin(i+64); } while(++i);
        break;
        case 2: // Square
            do { AWGTemp1[i] = (i<128)?127:-127; } while(++i);
        break;
        case 3: // Triangle
            for(;i<128;i++) AWGTemp1[i] = 127-((i)<<1);
            do { AWGTemp1[i] = -127+(i<<1); } while(++i);
        break;
        case 4: // Custom wave from EEPROM
            eeprom_read_block(AWGTemp1, EEwave, 256);
        break;
    }
    // Prepare buffer:
    // ******** Duty cycle ********
    i=0; j=0; step=0; d=512-AWG.duty;
    do {
        j=hibyte(step);
        AWGTemp2[j] = AWGTemp1[i];
        if(j<255) AWGTemp2[j+1] = (AWGTemp1[i]+AWGTemp1[i+1])/2;
        step+=d;
        if(i==127) d=AWG.duty;
    } while(++i);
    i=0;
    step = (12500000 / ( AWG.desiredF / AWG.cycles)) - 1;
    if(step<31) step=31;
    setbit(DMA.CH3.CTRLB,DMA_CH_TRNIF_bp); 
    while(!testbit(DMA.CH3.CTRLB,DMA_CH_TRNIF_bp));
    TCD0.PER = step;
    do {
    // ******** Multiply by Gain ********
        j=128+FMULS8(AWG.amp,AWGTemp2[(uint8_t)(i*AWG.cycles)]);
    // ******** Add Offset ********
        AWGBuffer[i]=addwsat(j,AWG.offset);
    } while(++i);
    // Set Timer
}
