/*****************************************************************************

Xprotolab - DEMO MSO FILE
Gabotronics C.A.
February 2011

Copyright 2011 Gabriel Anzziani

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#include <avr/io.h>
#include <util/delay_basic.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <math.h>
#include "main.h"
#include "mso.h"
#include "logic.h"
#include "awg.h"


// Function prototypes
uint8_t fft_stuff(uint8_t *p1);
void AutoCursorH(void);             // Automatically set horizontal cursors
void AutoCursorV(uint8_t n);        // Automatically set vertical the cursors
void Measurements(void);            // Measurements for Meter Mode
void ShowCursorV(void);             // Display Vertical Cursor
void ShowCursorH(uint8_t gain, uint8_t HcursorA, uint8_t HcursorB, int8_t CHPos);   // Display Horizontal Cursor
void LoadEE(void);                  // Load settings from EEPROM
void SaveEE(void);                  // Save settings to EEPROM
void Apply(void);                   // Apply oscilloscope settings

// Global variables
uint8_t  VcursorA  = 32;            // Vertical cursor A
uint8_t  VcursorB  = 96;            // Vertical cursor B
uint8_t  Hcursor1A = 96;            // CH1 Horizontal cursor A
uint8_t  Hcursor1B = 32;            // CH1 Horizontal cursor B
uint8_t  Hcursor2A = 96;            // CH2 Horizontal cursor A
uint8_t  Hcursor2B = 32;            // CH2 Horizontal cursor B
uint16_t slowval;                   // Slow sampling rate time value

ACHANNEL CH1,CH2;                   // Analog Channel 1, Channel 2, Channel 3
uint8_t fftm[128];                  // FFT magnitude data

// EEProm variables
uint8_t EEMEM EEsrate = 6;          // Sampling rate, start with 512uS/s
uint8_t EEMEM EEtdelay = 0;         // Trigger delay
uint8_t EEMEM EEtlevel = 128;       // Trigger level
uint8_t EEMEM EEtpre = 128;         // Number of Pre Trigger samples
uint8_t EEMEM EEtsource = 1;        // Trigger source, start with CH1
uint8_t EEMEM EECH1position = -96;  // CH1 vertical position
uint8_t EEMEM EECH1gain = 1;        // CH1 gain
uint8_t EEMEM EECH2position = -32;  // CH2 vertical position
uint8_t EEMEM EECH2gain = 1;        // CH2 gain
uint8_t EEMEM EECHDposition = 0;    // CHD vertical position
uint8_t EEMEM EECH1option= 0x01;    // CH1 on, 5V/div, no average
uint8_t EEMEM EECH2option= 0x01;    // CH2 on, 5V/div, no average
uint8_t EEMEM EECHDoption= 0xC4;    // CHD off, thick line when low, pull down
uint8_t EEMEM EECHDparam = 0x00;    // 
uint8_t EEMEM EECHDdecode = 0x00;   // 
uint8_t EEMEM EECHDmask = 0xFF;     // Mask logic inputs
uint8_t EEMEM EEMcursor = 0x00;     // Cursors off
uint8_t EEMEM EEMStatus = 0x01;     // Set free trigger
uint8_t EEMEM EEMset = 0x1C;        // Set scope mode, show grid, line
uint8_t EEMEM EEdisplay = 0x00;     // 
uint8_t EEMEM EEMFFT = 0x21;        // Use hamming window, no logarithm
uint8_t EEMEM EECHREF1[256] = {0};  // Reference waveform CH1
uint8_t EEMEM EECHREF2[256] = {0};  // Reference waveform CH2


// Offset calibration table (compensates OpAmp's offset voltage)
int8_t EEMEM offsetsCH1[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
int8_t EEMEM offsetsCH2[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

// ADC system clock timers

// TCE0 controls Interrupt ADC, srate: 7    8    9    10    11
const uint16_t TCE0val[4] PROGMEM = { 499, 499, 624, 1249};//, 2499 };

// TCE0 timer is fixed for slow sampling rates (50mS/div and under)
// slowcnt controls the sampling rate. 
const uint16_t slowcnt[11] PROGMEM = { 1,1,2,4,10,20,40,100,200,400,1000 };


// milivolts or volts per pixels * 100000
// Range                         5V   2.5V 1.25V 0.625V  312.5mV 156.25mV 78.125mV  39.0625
const uint32_t milivolts[8] = { 31250, 15625, 7812, 3906, 1953125, 976562, 488281, 781250 };

static const uint32_t freqval[22] PROGMEM = {
    // Kilo Hertz
    200000000,100000000,50000000,25000000,12500000,6250000,3125000,
    // Hertz
    1600000000,800000000,320000000,160000000,80000000,32000000,16000000,8000000,3200000,1600000,
               800000,   320000,   160000,   80000,   32000 };

const uint32_t timeval[22] PROGMEM = {   // = Time division * 10000 / 16
    /* micro Seconds */
    50000,100000,200000,400000,800000,1600000,3200000,
    /* mili Seconds */
    6250,12500,31250,62500,125000,312500,625000,1250000,3125000,
    /* Seconds */
    6250,12500,31250,62500,125000,312500 };


static char unitkHz[] PROGMEM = "KHZ";   // kHz
static char unitHz[]  PROGMEM = "HZ ";   // Hz
static char unitV[]   PROGMEM = " V  ";  // V
static char unitmV[]  PROGMEM = " &'V";  // mV

// Main MSO Application
void MSO(void) {

}

// Load settings from EEPROM
void LoadEE(void) {
    CH1.position = eeprom_read_byte(&EECH1position);  // Load CH1 vertical position
    CH1.option   = eeprom_read_byte(&EECH1option);    // Load CH1 options
    CH1.gain     = eeprom_read_byte(&EECH1gain);      // Load CH1 gain
    CH2.position = eeprom_read_byte(&EECH2position);  // Load CH2 vertical position
    CH2.option   = eeprom_read_byte(&EECH2option);    // Load CH2 options
    CH2.gain     = eeprom_read_byte(&EECH2gain);      // Load CH2 gain
    CHD.position = eeprom_read_byte(&EECHDposition);  // Load CHD vertical position
    CHD.option   = eeprom_read_byte(&EECHDoption);    // Load CHD options
    CHD.param    = eeprom_read_byte(&EECHDparam);     // Load CHD parameters
    CHD.decode   = eeprom_read_byte(&EECHDdecode);    // Load CHD parameters
    CHD.mask     = eeprom_read_byte(&EECHDmask);      // Load CHD mask
    srate        = eeprom_read_byte(&EEsrate);        // Load sampling rate
    Mcursor      = eeprom_read_byte(&EEMcursor);      // Load MSO options
    MStatus      = eeprom_read_byte(&EEMStatus);      // Load status
    MFFT         = eeprom_read_byte(&EEMFFT);         // Load FFT Options
    tdelay       = eeprom_read_byte(&EEtdelay);       // Load trigger delay
    tlevel       = eeprom_read_byte(&EEtlevel);       // Load trigger level
    tpre         = eeprom_read_byte(&EEtpre);         // Load # of trigger samples
    tsource      = eeprom_read_byte(&EEtsource);      // Load trigger source
    Mset         = eeprom_read_byte(&EEMset);         // Load mode and setting
    display      = eeprom_read_byte(&EEdisplay);      // Load display settings
}

// Save settings to EEPROM
void SaveEE(void) {
    eeprom_write_byte(&EECH1position, CH1.position);    // Save CH1 vertical position
    eeprom_write_byte(&EECH1option, CH1.option);        // Save CH1 options
    eeprom_write_byte(&EECH1gain, CH1.gain);            // Save CH1 gain
    eeprom_write_byte(&EECH2position, CH2.position);    // Save CH2 vertical position
    eeprom_write_byte(&EECH2option, CH2.option);        // Save CH2 options
    eeprom_write_byte(&EECH2gain, CH2.gain);            // Save CH2 gain
    eeprom_write_byte(&EECHDposition, CHD.position);    // Save CH2 vertical position
    eeprom_write_byte(&EECHDoption, CHD.option);        // Save CHD options
    eeprom_write_byte(&EECHDparam, CHD.param);          // Save CHD protocol options
    eeprom_write_byte(&EECHDdecode, CHD.decode);        // Save CHD decode select
    eeprom_write_byte(&EECHDmask, CHD.mask);            // Save CHD mask
    eeprom_write_byte(&EEsrate, srate);                 // Save sampling rate
    eeprom_write_byte(&EEMcursor, Mcursor);             // Save cursor settings
    eeprom_write_byte(&EEMStatus, MStatus);             // Save status
    eeprom_write_byte(&EEMset, Mset);                   // Save mode and settings
    eeprom_write_byte(&EEdisplay, display);             // Save display settings
    eeprom_write_byte(&EEMFFT, MFFT);                   // Save FFT Options
    eeprom_write_byte(&EEtdelay, tdelay);               // Save trigger delay
    eeprom_write_byte(&EEtlevel, tlevel);               // Save trigger level
    eeprom_write_byte(&EEtpre, tpre);                   // Save # of trigger samples
    eeprom_write_byte(&EEtsource, tsource);             // Save trigger source
}

// Timer for auto trigger and auto key repeat
ISR(TCC1_OVF_vect) {
    // Trigger for AUTO mode:
    if(!testbit(MStatus, stop) && testbit(MStatus, autotrg)) {
        if(!key && !testbit(MStatus, trigger)) {    // To prevent updates with low sampling rate
            setbit(Signals, update);
            setbit(MStatus, trigger);
        }
    }
    if (key==KBR || key==KCR) keyrep++;
}

